/*

  DFPlayer Mini MP3 Module

  Parts required:
  - Arduino Board
  - DFPlayer Mini MP3 Module
  - Resistor 1K
  - Speaker or earphones
  - Micro SD Card

  Libraries required:
  - SoftwareSerial (Included in Arduino IDE)

  Programming by : René Rasmussen
  Website : www.it-teknolog.dk
  Contact : renerasmussen@it-teknolog.dk
  Date : 7 Apr 2020
  Version : v1.0

*/

#include "SoftwareSerial.h"                                                                 // include library SoftwareSerial
SoftwareSerial mySerial(10, 11);                                                            // create object from SoftwareSerial to mySerial and config digital ports

# define Start_Byte 0x7E                                                                    // define variables for MP3 Module
# define Version_Byte 0xFF
# define Command_Length 0x06
# define End_Byte 0xEF
# define Acknowledge 0x00
# define ACTIVATED LOW

boolean isPlaying = false;                                                                  // set variable to boolean and set value to false

void setup () {
  mySerial.begin (9600);                                                                    // start mySerial communication on 9600 baud
  delay(1000);                                                                              // wait 1000 ms

  playFirst();                                                                              // execute function playFirst()
  setVolume(0x15);                                                                          // execute function setVolume() with variable 0x15
  
  // --- Ekstra Features ---
  //pause();                                                                                // execute function pause() to pause 
  //play();                                                                                 // execute function play() to play MP3  
  //playNext();                                                                             // execute function playNext() to play next MP3
  //playPrevious();                                                                         // execute function playPrevious() to play previous MP3
  
}
void loop () {
}

void playFirst()                                                                            // function playFirst()
{
  execute_CMD(0x3F, 0, 0);                                                                  // send data to function execute_CMD
  delay(500);                                                                               // wait 500ms
  setVolume(20);                                                                            // send data to setVolume 
  delay(500);                                                                               // wait 500ms
  execute_CMD(0x11, 0, 1);                                                                  // send data to function execute_CMD
  delay(500);                                                                               // wait 500ms
}
void pause()                                                                                // function pause()
{
  execute_CMD(0x0E, 0, 0);                                                                  // send data to function execute_CMD
  delay(500);                                                                               // wait 500ms
}
void play()                                                                                 // function play()
{
  execute_CMD(0x0D, 0, 1);                                                                  // send data to function execute_CMD
  delay(500);                                                                               // wait 500ms
}
void playNext()                                                                             // function playNext()
{
  execute_CMD(0x01, 0, 1);                                                                  // send data to function execute_CMD
  delay(500);                                                                               // wait 500ms
}
void playPrevious()                                                                         // function playPrevious()
{
  execute_CMD(0x02, 0, 1);                                                                  // send data to function execute_CMD
  delay(500);                                                                               // wait 500ms
}
void setVolume(int volume)                                                                  // function setVolume(int volume)
{
  execute_CMD(0x06, 0, volume); // Set the volume (0x00~0x30)                               // send data to function execute_CMD
  delay(2000);                                                                              // wait 2000ms
}
void execute_CMD(byte CMD, byte Par1, byte Par2) {                                          // function execute_CMD(byte CMD, byte Par1, byte Par2)
  word checksum = -(Version_Byte + Command_Length + CMD + Acknowledge + Par1 + Par2);       // calculate checksum 2 bytes
 
  byte Command_line[10] = {                                                                 // build the command line
    Start_Byte, Version_Byte, Command_Length, CMD, Acknowledge,
    Par1, Par2, highByte(checksum), lowByte(checksum), End_Byte
  };
 
  for (byte k = 0; k < 10; k++)
  {
    mySerial.write( Command_line[k]);                                                       // send the command line to the module
  }
}
